# -*- perl -*-

# Copyright (c) 2002 by Jeff Weisberg
# Author: Jeff Weisberg <argus @ tcp4me.com>
# Date: 2002-Nov-01 15:57 (EST)
# Function: misc graph data handling
#
# $Id: GraphData.pm,v 1.6 2002/12/30 23:43:38 jaw Exp $

package GraphData;
require "graph.ph";

sub new {
    my $c = shift;
    my $name = shift;
    my( $fh, $buf, $magic );
    my $me = {
	name => $name,
    };

    $fh = anon_fh();
    $me->{fd}= $fh;
    open( $fh, "$::datadir/gdata/$name" )
	|| return ::error( "'$name' is stubborn and refuses to open: $!" );
    binmode $fh;
    
    # read file header
    seek($fh, 0, 0);
    read($fh, $buf, $HDR_SIZE);

    ($magic, $me->{lastt},
     $me->{sampl_count}, $me->{sampl_index},
     $me->{hours_count}, $me->{hours_index},
     $me->{days_count},  $me->{days_index},
     ) = unpack( "a4N NN NN NN", $buf );

    unless( $magic eq $MAGIC){
	return ::error( "corrupt data file: $name" );
    }

    bless $me;
}

# But what Gherardo is it, who, as sample
# Of a lost race, thou sayest has remained
#   -- Dante, Divine Comedy
sub readsamples {
    my $me = shift;
    my( $fh, $start, $i, @samp );

    $fh = $me->{fd};
    
    $start = ($me->{sampl_index} - $me->{sampl_count} + $SAMP_NMAX) % $SAMP_NMAX;
    # print STDERR "samples=$me->{sampl_count} index=$me->{sampl_index} start=$start\n";
    for( $i=0; $i<$me->{sampl_count}; $i++ ){
	my $n = ($i + $start) % $SAMP_NMAX;
	my $off = $n * $SAMP_SIZE + $SAMP_START;
	my $buf;
	seek( $fh, $off, 0 );
	read( $fh, $buf, $SAMP_SIZE );
	my( $t, $v, $f ) =
	    unpack( "NfN", $buf );
	next unless $t;
	push @samp, {
	    time => $t,
	    valu => $v,
	    flag => $f,
	};
    }

    # remove end-errors
    shift @samp
	if( $samp[0]{time} > $samp[1]{time} );
    
    # temporal limit
    my $et = $samp[-1]{time};
    while( $et - $samp[0]{time} > (36 * 3600) ){
	shift @samp;
    }

    $me->{samples} = [ @samp ];
}

# Here's the scroll,
# The continent and summary of my fortune
#   -- Shakespeare, Merchant of Venice
sub readsummary {
    my $me = shift;
    my $wh = shift;
    my $ns = shift;
    my( $fh, $start, $first, $cnt, $idx, @samp );

    $fh = $me->{fd};
    # day+hours data is same, different offset
    if( $wh eq 'hours' ){
	$cnt = $me->{hours_count};
	$idx = $me->{hours_index};
	$start = $HOURS_START;
    }else{
	$cnt = $me->{days_count};
	$idx = $me->{days_index};
	$start = $DAYS_START;
    }
    $cnt = $ns if $cnt > $ns;
    
    $first = ($idx - $cnt + $HOURS_NMAX) % $HOURS_NMAX;

    for( $i=0; $i<$cnt; $i++ ){
	my $n = ($i + $first) % $HOURS_NMAX;
	my $off = $n * $HOURS_SIZE + $start;
	my $buf;

	seek( $fh, $off, 0 );
	read( $fh, $buf, $HOURS_SIZE );
	my( $t, $min, $max, $ave, $sdv, $ns, $fl
	    ) = unpack( "NffffNNx4", $buf );
	next unless $t;
	next unless $ns > 1;	# NB: otherwise the graphs get ugly
	push @samp, {
	    time => $t,
	    flag => $fl,
	    min  => $min, max  => $max,
	    ave  => $ave, stdv => $sdv,
	    ns   => $ns,
	};
    }

    shift @samp
	if( $samp[0]{time} > $samp[1]{time} );
    
    # temporal limit
    my $et = $samp[-1]{time};
    my $max = ($which eq 'hours') ? 180 * 3600 : 90 * 24 * 3600;
    while( $et - $samp[0]{time} > $max ){
	shift @samp;
    }
    
    $me->{samples} = [ @samp ];
}


sub anon_fh {
    do { local *FILEHANDLE };
}

1;

