#!__PERL__
# -*- perl -*-

# Copyright (c) 2002 by Jeff Weisberg
# Author: Jeff Weisberg <argus @ tcp4me.com>
# Date: 2002-Oct-31 15:45 (EST)
# Function: server side of graph data collector
#
# $Id: graphd.pl,v 1.5 2002/12/30 23:43:41 jaw Exp $

# read in data from argusd, save to data files
# uses a large number of file descriptors, be sure
# that the system limit is sufficient (eg. limit/ulimit openfile/maxfiles/...)


use lib('__LIBDIR__');
require "conf.pl";
require "graph.ph";
use Sys::Syslog qw(:DEFAULT setlogsock);

%obj = ();

$syslog = shift @ARGV;
if( $syslog ){
    eval { setlogsock('unix'); };
    openlog( 'graphd', 'pid ndelay', $syslog );
}
loggit( "restarted" );

while( <> ){
    chop;
    if( /^sample/ ){
	# sample T obj st val
	my(undef, $t, $name, $st, $val
	   ) = split;
	newobj( $name ) unless defined $obj{$name};
	add_sample( $name, $t, $st, $val );
    }
}

# It adds a precious seeing to the eye
#   -- Shakespeare, Love's Labour 's Lost
sub add_sample {
    my $name = shift;
    my $time = shift;
    my $stat = shift;
    my $valu = shift;
    my( $me, $fh, $off, $flg );

    $me = $obj{$name};
    $fh = $me->{fd};

    # updating too fast?
    return if $time - $me->{lastt} < 30;
    
    $off = $me->{sampl_index} * $SAMP_SIZE + $SAMP_START;
    sysseek( $fh, $off, 0 );
    syswrite($fh, pack("NfNx4",
		       $time, $valu, flags($stat)
		       ), $SAMP_SIZE);
    
    $me->{sampl_index} ++;
    $me->{sampl_index} %= $SAMP_NMAX;
    $me->{sampl_count} ++
	unless $me->{sampl_count} >= $SAMP_NMAX;

    # update hrly, daily, ...
    $me->{hours_min} = $valu
	if !$me->{hours_nsamp} || ($valu < $me->{hours_min});
    $me->{hours_max} = $valu
	if !$me->{hours_nsamp} || ($valu > $me->{hours_max});
    $me->{hours_sigma} += $valu;
    $me->{hours_sigm2} += $valu * $valu;
    $me->{hours_nsamp} ++;
    $me->{hours_flags} = sum_flags( $me->{hours_flags}, $stat );
    
    $me->{days_min} = $valu
	if !$me->{days_nsamp}  || ($valu < $me->{days_min});
    $me->{days_max} = $valu
	if !$me->{days_nsamp}  || ($valu > $me->{days_max});
    $me->{days_sigma} += $valu;
    $me->{days_sigm2} += $valu * $valu;
    $me->{days_nsamp} ++;
    $me->{days_flags} = sum_flags( $me->{days_flags}, $stat );

    # roll?
    my( @lt, @ct );
    @lt = localtime $me->{lastt};
    @ct = localtime $time;
    if( $me->{lastt} && ($lt[2] != $ct[2]) ){
	# add hour data
	my($ave, $std) = ave_sdev( $me->{hours_sigma},
				   $me->{hours_sigm2},
				   $me->{hours_nsamp});
	$off = $me->{hours_index} * $HOURS_SIZE + $HOURS_START;
	sysseek( $fh, $off, 0 );
	syswrite($fh, pack("NffffNNx4",
			   $time,
			   $me->{hours_min}, $me->{hours_max},
			   $ave, $std, 
			   $me->{hours_nsamp},
			   $me->{hours_flags},
			   ), $HOURS_SIZE);
	$me->{hours_index} ++;
	$me->{hours_index} %= $HOURS_NMAX;
	$me->{hours_count} ++
	    unless $me->{hours_count} >= $HOURS_NMAX;

	# reset stats
	$me->{hours_min} = $me->{hours_max} = 
	    $me->{hours_nsamp} = $me->{hours_sigma} = 
	    $me->{hours_sigm2} = $me->{hours_flags} = 0;
    }

    if( $me->{lastt} && ($lt[3] != $ct[3]) ){
	# add day data
	my($ave, $std) = ave_sdev( $me->{days_sigma},
				   $me->{days_sigm2},
				   $me->{days_nsamp});
	$off = $me->{days_index} * $DAYS_SIZE + $DAYS_START;
	sysseek( $fh, $off, 0 );
	syswrite($fh, pack("NffffNNx4",
			   $time,
			   $me->{days_min}, $me->{days_max},
			   $ave, $std, 
			   $me->{days_nsamp},
			   $me->{days_flags},
			   ), $DAYS_SIZE);

	$me->{days_index} ++;
	$me->{days_index} %= $DAYS_NMAX;
	$me->{days_count} ++
	    unless $me->{days_count} >= $DAYS_NMAX;

	# reset stats
	$me->{days_min} = $me->{days_max} =
	    $me->{days_nsamp} = $me->{days_sigma} =
	    $me->{days_sigm2} = $me->{days_flags} = 0;
    }
    
    # QQQ update header?
    update_hdr( $me );

    $me->{lastt} = $time;
}

# Yet, for necessity of present life,
# I must show out a flag and sign of love,
# Which is indeed but sign. That you shall surely find him,
#   -- Shakespeare, Othello
sub flags {
    my $st = shift;

    {
	up => 0,
	down => 1,
	override => 2,
    }->{$st};
}

sub sum_flags {
    my $o = shift;
    my $n = flags( shift );

    return 1 if $o == 1 || $n == 1;
    return 2 if $o == 2 || $n == 2;
    0;
}

sub update_hdr {
    my $me = shift;
    my $fh = $me->{fd};
    my $hdr;

    $hdr = $MAGIC . pack( "N NN NN NN ffNfdNx4 ffNfdNx4",
			  $me->{lastt},
			  $me->{sampl_count}, $me->{sampl_index},
			  $me->{hours_count}, $me->{hours_index},
			  $me->{days_count},  $me->{days_index},

			  $me->{hours_min},   $me->{hours_max},
			  $me->{hours_nsamp}, $me->{hours_sigma},
			  $me->{hours_sigm2}, $me->{hours_flags},

			  $me->{days_min},   $me->{days_max},
			  $me->{days_nsamp}, $me->{days_sigma},
			  $me->{days_sigm2}, $me->{days_flags},
			  );
    sysseek( $fh, 0, 0 );
    syswrite($fh, $hdr );
}

sub newobj {
    my $name = shift;
    my( $fh, $me );
    
    # print STDERR "GRAPHD: new $name\n";

    $me = $obj{$name} = {
	name => $name,
    };

    $fh = anon_fh();
    $me->{fd} = $fh;

    # XXX - maxfiles? logmsgs?
    if( -f "$datadir/gdata/$name" ){
	open( $fh, "+< $datadir/gdata/$name" )
	    || loggit( "$name on too tight, won't open: $!" );
	initstats($me);
    }else{
	open( $fh, "+> $datadir/gdata/$name" )
	    || loggit( "$name stuck, won't open: $!" );
	initfile($me);
    }
    binmode $fh;
}

sub initfile {
    my $me = shift;
    my $hdr = $MAGIC . ("\0" x 1020);
    my $fh = $me->{fd};

    loggit( "creating data file $me->{name}" );
    sysseek( $fh, 0, 0 );
    syswrite($fh, $hdr, $HDR_SIZE);

    # pre-extend
    my($x) = $DAYS_START + $DAYS_SIZE * $DAYS_NMAX - 1;
    sysseek( $fh, $x, 0 );
    syswrite($fh, "\0", 1);
}

sub initstats {
    my $me = shift;
    my $fh = $me->{fd};
    my( $buf, $magic );
    
    sysseek($fh, 0, 0);
    sysread($fh, $buf, $HDR_SIZE);

    ($magic, $me->{lastt},
     $me->{sampl_count}, $me->{sampl_index},
     $me->{hours_count}, $me->{hours_index},
     $me->{days_count},  $me->{days_index},
     
     $me->{hours_min},   $me->{hours_max},
     $me->{hours_nsamp}, $me->{hours_sigma},
     $me->{hours_sigm2}, $me->{hours_flags},
     
     $me->{days_min},   $me->{days_max},
     $me->{days_nsamp}, $me->{days_sigma},
     $me->{days_sigm2}, $me->{days_flags},
     ) = unpack( "a4N NN NN NN ffNfdNx4 ffNfdNx4", $buf );

    unless( $magic eq $MAGIC){
	loggit( "corrupt data file: $me->{name}" );
	close $fh;
    }
}

sub ave_sdev {
    my $s  = shift;
    my $s2 = shift;
    my $n  = shift;
    my( $u, $v );

    # things may overflow and we may try taking sqrt of a negative
    # (like if some wiseguy tries graphing uptime)
    eval {
	$u  = $s / $n;
	$v  = sqrt( $s2 / $n - $u * $u );
    };
    if( $@ ){
	$v = 0;
    }
    ($u, $v);
}

sub anon_fh {
    do { local *FILEHANDLE };
}

sub loggit {
    my $msg = shift;

    eval {
	syslog( 'info', $msg )  if $syslog;
    };
    print STDERR "GRAPHD: $msg\n";
}


# file: header, samples, hr data, day data
# header: 1k {long: magic, T, Nsample, sampleindx, Nhrs, hrindx, Ndays, dayindx, ...}
# samples: 2k x 16 bytes { time, value, flags, 0 }

