#!__PERL__
# -*- perl -*-

# Copyright (c) 2002 by Jeff Weisberg
# Author: Jeff Weisberg <argus @ tcp4me.com>
# Date: 2002-Apr-02 12:09 (EST)
# Function: main driver program
#
# $Id: main.pl,v 1.29 2003/01/17 04:28:37 jaw Exp jaw $


use lib('__LIBDIR__');
BEGIN{require "conf.pl";}

use Doc;
use Control;
use BaseIO;
use Cron;
use Server;
use Commands;
use Conf;
use Configable;
use MonEl;
use Group;
use Alias;
use Service;
use Error;
use Notify;
use Graph;
use NullConf;
require "misc.pl";
require "encode.pl";
require "loctime.pl";
BEGIN {
    # these may or may not be present
    eval { require Tkt; };			# ticket system interface
    eval { require DARP; $HAVE_DARP = 1; };	# distributed argus redundancy protocol
}


use Sys::Syslog qw(:DEFAULT setlogsock);
use Getopt::Std;

getopts("a:c:dfg:htu:CDEHT");

$MAX_PERIOD = 300;
$NAME = "Argus";

$mainpid  = 0;
$starttime = 0;
$mainstart = $^T;
($OSINFO = join(' ', (`uname -s`, `uname -r`, `uname -m`)))
    =~ s/[\n\r]//g;

my $childpid = 0;
my $exitinprogress = 0;

if( $opt_h ){
    print <<U;
usage: $0 [options]
	 -f    keep stdio open and run in foreground
	 -d    debugging
	 -a alt. data dir
	 -c alt. config file
	 -u user    change to the specified user at startup
	 -g group   change to the specified user at startup

    the following options will perform their function and exit
	 -h    display a message not entirely unlike this one
	 -D    describe config file
	 -E    describe internal data structures
	 -C    describe control channel commands
	 -H    make above descriptions in html
	 -t    syntax check config files
	 -T    syntax check config files, and more...
U
    ;
    exit;
}

$datadir = $opt_a if $opt_a;
$opt_f ||= $opt_d;
$opt_t ||= $opt_T;

Doc::describe_config( $opt_H )  if $opt_D;
Doc::describe_internal( $opt_H )if $opt_E;
Doc::describe_control( $opt_H ) if $opt_C;

if( $opt_t ){
    Conf::readconfig( $opt_c || "$datadir/config" );
    exit $Conf::has_errors;
}

# change uid/gid
if( $opt_g ){
    my $gid = (getgrnam($opt_g))[2];
    $gid = $opt_g if( !defined($gid) && $opt_g =~ /^\d+$/ );
    die "invalid group for -g option. aborting.\n" unless defined $gid;
    $( = $gid; $) = $gid;
}
if( $opt_u ){
    my $uid = (getpwnam($opt_u))[2];
    $uid = $opt_u if( !defined($uid) && $opt_u =~ /^\d+$/ );
    die "invalid user for -u option. aborting.\n" unless defined $uid;
    $! = 0;
    $< = $> = $uid;
    die "unable to change uid: $!\n" if $!;
}


# daemonize
unless( $opt_f ){
    # The victor daemon mounts obscure in air, 
    # While the ship sails without the pilot's care.
    #   -- Virgil, Aeneid
    close STDIN;
    close STDOUT;
    close STDERR;
    open( STDIN,  "/dev/null" );
    open( STDOUT, "/dev/null" );
    open( STDERR, "/dev/null" );
    fork && exit;
}

$SIG{PIPE} = 'IGNORE';
$SIG{HUP} = $SIG{QUIT} = $SIG{INT} = $SIG{TERM} = sub {
    loggit( "parent caught signal SIG$_[0] - exiting", 1 );
    if( $childpid > 1 ){
	kill "TERM", $childpid;
	wait;
    }
    exit unless $_[0] eq 'HUP';
};

$0 = "$NAME";

while(1){

    $^T = time();
    $mainpid = $$;
    if( !$opt_f && ($childpid = fork) ){
        # parent
	# parent blocks in wait, then reforks child
        while( 1 ){
            my $i;
            $i = waitpid $childpid, 0;
            next if( $i == -1 && $! == 4 );
            last;
        }
        next;
    }
    # child
    $SIG{HUP} = $SIG{QUIT} = $SIG{INT} = $SIG{TERM} = sub {
	loggit( "child caught signal SIG$_[0] - exiting", 1 );
	return if $exitinprogress;
	$exitinprogress = 1;
	exit;
    };
    $SIG{ALRM} = sub {};
    
    Conf::readconfig( $opt_c || "$datadir/config" );
    xxx_openlog();
    Doc::check_all();
    # Unto me, saying, "Brother, look and listen!"
    #     -- Dante Divine Comedy
    Control->Server::new_local( "$datadir/control", oct($Conf::Config{chmod_control}) );
    DARP   ->Server::new_inet( 6543 )	# XXX
	if $HAVE_DARP;
    system( $Conf::Config{runatstartup} ) if $Conf::Config{runatstartup};
    
    $^T = time();
    $starttime = $^T;
    loggit( "successful restart - $NAME running", 1 );

    BaseIO::mainloop(
		     maxperiod => $MAX_PERIOD,
		     # run       => \&Prog::reap,
		     );
    
}

